@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 2006 Free Software Foundation, Inc.
+   Copyright (C) 2006, 2010, 2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,21 +15,25 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
-#include "any-reader.h"
+
+#include "data/any-reader.h"
+
 #include <assert.h>
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <libpspp/assertion.h>
-#include <libpspp/message.h>
-#include "file-handle-def.h"
-#include "file-name.h"
-#include "por-file-reader.h"
-#include "sys-file-reader.h"
-#include <libpspp/str.h>
-#include "scratch-reader.h"
-#include "xalloc.h"
+
+#include "data/file-handle-def.h"
+#include "data/file-name.h"
+#include "data/por-file-reader.h"
+#include "data/scratch-reader.h"
+#include "data/sys-file-reader.h"
+#include "libpspp/assertion.h"
+#include "libpspp/message.h"
+#include "libpspp/str.h"
+
+#include "gl/xalloc.h"
 
 #include "gettext.h"
 #define _(msgid) gettext (msgid)
@@ -42,30 +46,38 @@ enum detect_result
     IO_ERROR                    /* File couldn't be opened. */
   };
 
-/* Tries to detect whether HANDLE represents a given type of
-   file, by opening the file and passing it to DETECT, and
-   returns a detect_result. */
+/* Tries to detect whether FILE is a given type of file, by opening the file
+   and passing it to DETECT, and returns a detect_result. */
 static enum detect_result
-try_detect (struct file_handle *handle, bool (*detect) (FILE *))
+try_detect (const char *file_name, bool (*detect) (FILE *))
 {
   FILE *file;
   bool is_type;
 
-  file = fn_open (fh_get_file_name (handle), "rb");
+  file = fn_open (file_name, "rb");
   if (file == NULL)
     {
-      msg (ME, _("An error occurred while opening \"%s\": %s."),
-           fh_get_file_name (handle), strerror (errno));
+      msg (ME, _("An error occurred while opening `%s': %s."),
+           file_name, strerror (errno));
       return IO_ERROR;
     }
 
   is_type = detect (file);
 
-  fn_close (fh_get_file_name (handle), file);
+  fn_close (file_name, file);
 
   return is_type ? YES : NO;
 }
 
+/* Returns true if any_reader_open() would be able to open FILE as a data
+   file, false otherwise. */
+bool
+any_reader_may_open (const char *file)
+{
+  return (try_detect (file, sfm_detect) == YES
+          || try_detect (file, pfm_detect) == YES);
+}
+
 /* Returns a casereader for HANDLE.  On success, returns the new
    casereader and stores the file's dictionary into *DICT.  On
    failure, returns a null pointer. */
@@ -78,19 +90,19 @@ any_reader_open (struct file_handle *handle, struct dictionary **dict)
       {
         enum detect_result result;
 
-        result = try_detect (handle, sfm_detect);
+        result = try_detect (fh_get_file_name (handle), sfm_detect);
         if (result == IO_ERROR)
           return NULL;
         else if (result == YES)
           return sfm_open_reader (handle, dict, NULL);
 
-        result = try_detect (handle, pfm_detect);
+        result = try_detect (fh_get_file_name (handle), pfm_detect);
         if (result == IO_ERROR)
           return NULL;
         else if (result == YES)
           return pfm_open_reader (handle, dict, NULL);
 
-        msg (SE, _("\"%s\" is not a system or portable file."),
+        msg (SE, _("`%s' is not a system or portable file."),
              fh_get_file_name (handle));
         return NULL;
       }
