@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-9, 2000, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000, 2006, 2010, 2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -20,24 +20,29 @@
 
 #include <ctype.h>
 #include <stdlib.h>
+#include <uniwidth.h>
 
-#include <data/identifier.h>
-#include <data/settings.h>
-#include <data/value.h>
-#include <data/variable.h>
-#include <libpspp/assertion.h>
-#include <libpspp/compiler.h>
-#include <libpspp/message.h>
-#include <libpspp/misc.h>
-#include <libpspp/str.h>
+#include "data/identifier.h"
+#include "data/settings.h"
+#include "data/value.h"
+#include "data/variable.h"
+#include "libpspp/assertion.h"
+#include "libpspp/cast.h"
+#include "libpspp/compiler.h"
+#include "libpspp/message.h"
+#include "libpspp/misc.h"
+#include "libpspp/str.h"
 
-#include "minmax.h"
-#include "xalloc.h"
+#include "gl/minmax.h"
+#include "gl/xalloc.h"
 
 #include "gettext.h"
 #define _(msgid) gettext (msgid)
 
-
+struct fmt_settings
+  {
+    struct fmt_number_style styles[FMT_NUMBER_OF_FORMATS];
+  };
 
 bool is_fmt_type (enum fmt_type);
 
@@ -45,35 +50,115 @@ static bool valid_width (enum fmt_type, int width, bool for_input);
 
 static int max_digits_for_bytes (int bytes);
 
-void fmt_number_style_init (struct fmt_number_style *style);
+static void fmt_affix_set (struct fmt_affix *, const char *);
+static void fmt_affix_free (struct fmt_affix *);
 
+static void fmt_number_style_init (struct fmt_number_style *);
+static void fmt_number_style_clone (struct fmt_number_style *,
+                                    const struct fmt_number_style *);
+static void fmt_number_style_destroy (struct fmt_number_style *);
 
-/* Initialize the format module. */
-struct fmt_number_style *
-fmt_create (void)
+/* Creates and returns a new struct fmt_settings with default format styles. */
+struct fmt_settings *
+fmt_settings_create (void)
 {
-  struct fmt_number_style *styles =
-    xcalloc (FMT_NUMBER_OF_FORMATS, sizeof (*styles));
-
+  struct fmt_settings *settings;
   int t;
+
+  settings = xzalloc (sizeof *settings);
   for (t = 0 ; t < FMT_NUMBER_OF_FORMATS ; ++t )
-    fmt_number_style_init (&styles[t]);
+    fmt_number_style_init (&settings->styles[t]);
+  fmt_settings_set_decimal (settings, '.');
 
-  fmt_set_decimal (styles, '.');
-
-  return styles;
+  return settings;
 }
 
-
-/* Deinitialize the format module. */
+/* Destroys SETTINGS. */
 void
-fmt_done (struct fmt_number_style *styles)
+fmt_settings_destroy (struct fmt_settings *settings)
 {
-  int t;
-  for (t = 0 ; t < FMT_NUMBER_OF_FORMATS ; ++t )
-    fmt_number_style_destroy (&styles[t]);
+  if (settings != NULL)
+    {
+      int t;
 
-  free (styles);
+      for (t = 0 ; t < FMT_NUMBER_OF_FORMATS ; ++t )
+        fmt_number_style_destroy (&settings->styles[t]);
+
+      free (settings->styles);
+    }
+}
+
+/* Returns a copy of SETTINGS. */
+struct fmt_settings *
+fmt_settings_clone (const struct fmt_settings *old)
+{
+  struct fmt_settings *new;
+  int t;
+
+  new = xmalloc (sizeof *new);
+  for (t = 0 ; t < FMT_NUMBER_OF_FORMATS ; ++t )
+    fmt_number_style_clone (&new->styles[t], &old->styles[t]);
+
+  return new;
+}
+
+/* Returns the number formatting style associated with the given
+   format TYPE. */
+const struct fmt_number_style *
+fmt_settings_get_style (const struct fmt_settings *settings,
+                        enum fmt_type type)
+{
+  assert (is_fmt_type (type));
+  return &settings->styles[type];
+}
+
+/* Sets the number style for TYPE to have the given DECIMAL and GROUPING
+   characters, negative prefix NEG_PREFIX, prefix PREFIX, suffix SUFFIX, and
+   negative suffix NEG_SUFFIX.  All of the strings are UTF-8 encoded. */
+void
+fmt_settings_set_style (struct fmt_settings *settings, enum fmt_type type,
+                        char decimal, char grouping,
+                        const char *neg_prefix, const char *prefix,
+                        const char *suffix, const char *neg_suffix)
+{
+  struct fmt_number_style *style = &settings->styles[type];
+  int total_bytes, total_width;
+
+  assert (grouping == '.' || grouping == ',' || grouping == 0);
+  assert (decimal == '.' || decimal == ',');
+  assert (decimal != grouping);
+
+  fmt_number_style_destroy (style);
+
+  fmt_affix_set (&style->neg_prefix, neg_prefix);
+  fmt_affix_set (&style->prefix, prefix);
+  fmt_affix_set (&style->suffix, suffix);
+  fmt_affix_set (&style->neg_suffix, neg_suffix);
+  style->decimal = decimal;
+  style->grouping = grouping;
+
+  total_bytes = (strlen (neg_prefix) + strlen (prefix)
+                 + strlen (suffix) + strlen (neg_suffix));
+  total_width = (style->neg_prefix.width + style->prefix.width
+                 + style->suffix.width + style->neg_suffix.width);
+  style->extra_bytes = MAX (0, total_bytes - total_width);
+}
+
+/* Sets the decimal point character for the settings in S to DECIMAL.
+
+   This has no effect on custom currency formats. */
+void
+fmt_settings_set_decimal (struct fmt_settings *s, char decimal)
+{
+  int grouping = decimal == '.' ? ',' : '.';
+  assert (decimal == '.' || decimal == ',');
+
+  fmt_settings_set_style (s, FMT_F,      decimal,        0, "-",  "",  "", "");
+  fmt_settings_set_style (s, FMT_E,      decimal,        0, "-",  "",  "", "");
+  fmt_settings_set_style (s, FMT_COMMA,  decimal, grouping, "-",  "",  "", "");
+  fmt_settings_set_style (s, FMT_DOT,   grouping,  decimal, "-",  "",  "", "");
+  fmt_settings_set_style (s, FMT_DOLLAR, decimal, grouping, "-", "$",  "", "");
+  fmt_settings_set_style (s, FMT_PCT,    decimal,        0, "-",  "", "%", "");
 }
 
 /* Returns an input format specification with type TYPE, width W,
@@ -772,15 +857,16 @@ fmt_to_io (enum fmt_type type)
 bool
 fmt_from_io (int io, enum fmt_type *fmt_type)
 {
-  enum fmt_type type;
-
-  for (type = 0; type < FMT_NUMBER_OF_FORMATS; type++)
-    if (get_fmt_desc (type)->io == io)
-      {
-        *fmt_type = type;
-        return true;
-      }
-  return false;
+  switch (io)
+    {
+#define FMT(NAME, METHOD, IMIN, OMIN, IO, CATEGORY)     \
+    case IO:                                          \
+      *fmt_type = FMT_##NAME;                           \
+      return true;
+#include "format.def"
+    default:
+      return false;
+    }
 }
 
 /* Returns true if TYPE may be used as an input format,
@@ -856,114 +942,75 @@ max_digits_for_bytes (int bytes)
   return map[bytes - 1];
 }
 
+/* Sets AFFIX's string value to S, a UTF-8 encoded string. */
+static void
+fmt_affix_set (struct fmt_affix *affix, const char *s)
+{
+  affix->s = s[0] == '\0' ? CONST_CAST (char *, "") : xstrdup (s);
+  affix->width = u8_strwidth (CHAR_CAST (const uint8_t *, s), "UTF-8");
+}
 
+/* Frees data in AFFIX. */
+static void
+fmt_affix_free (struct fmt_affix *affix)
+{
+  if (affix->s[0])
+    free (affix->s);
+}
 
-void
+static void
 fmt_number_style_init (struct fmt_number_style *style)
 {
-  style->neg_prefix = ss_empty ();
-  style->prefix = ss_empty ();
-  style->suffix = ss_empty ();
-  style->neg_suffix = ss_empty ();
+  fmt_affix_set (&style->neg_prefix, "");
+  fmt_affix_set (&style->prefix, "");
+  fmt_affix_set (&style->suffix, "");
+  fmt_affix_set (&style->neg_suffix, "");
   style->decimal = '.';
   style->grouping = 0;
 }
 
+static void
+fmt_number_style_clone (struct fmt_number_style *new,
+                        const struct fmt_number_style *old)
+{
+  fmt_affix_set (&new->neg_prefix, old->neg_prefix.s);
+  fmt_affix_set (&new->prefix, old->prefix.s);
+  fmt_affix_set (&new->suffix, old->suffix.s);
+  fmt_affix_set (&new->neg_suffix, old->neg_suffix.s);
+  new->decimal = old->decimal;
+  new->grouping = old->grouping;
+  new->extra_bytes = old->extra_bytes;
+}
 
 /* Destroys a struct fmt_number_style. */
-void
+static void
 fmt_number_style_destroy (struct fmt_number_style *style)
 {
   if (style != NULL)
     {
-      ss_dealloc (&style->neg_prefix);
-      ss_dealloc (&style->prefix);
-      ss_dealloc (&style->suffix);
-      ss_dealloc (&style->neg_suffix);
+      fmt_affix_free (&style->neg_prefix);
+      fmt_affix_free (&style->prefix);
+      fmt_affix_free (&style->suffix);
+      fmt_affix_free (&style->neg_suffix);
     }
 }
 
-/* Returns the number formatting style associated with the given
-   format TYPE. */
-const struct fmt_number_style *
-fmt_get_style (const struct fmt_number_style *styles, enum fmt_type type)
-{
-  assert (is_fmt_type (type));
-  return &styles[type];
-}
-
-
-/* Checks that style is STYLE sane */
-void
-fmt_check_style (const struct fmt_number_style *style)
-{
-  assert (ss_length (style->neg_prefix) <= FMT_STYLE_AFFIX_MAX);
-  assert (ss_length (style->prefix) <= FMT_STYLE_AFFIX_MAX);
-  assert (ss_length (style->suffix) <= FMT_STYLE_AFFIX_MAX);
-  assert (ss_length (style->neg_suffix) <= FMT_STYLE_AFFIX_MAX);
-  assert (style->decimal == '.' || style->decimal == ',');
-  assert (style->grouping == '.' || style->grouping == ','
-          || style->grouping == 0);
-  assert (style->grouping != style->decimal);
-}
-
-
-/* Returns the total width of the standard prefix and suffix for
-   STYLE. */
+/* Returns the total width of the standard prefix and suffix for STYLE, in
+   display columns (e.g. as returned by u8_strwidth()). */
 int
 fmt_affix_width (const struct fmt_number_style *style)
 {
-  return ss_length (style->prefix) + ss_length (style->suffix);
+  return style->prefix.width + style->suffix.width;
 }
 
-/* Returns the total width of the negative prefix and suffix for
-   STYLE. */
+/* Returns the total width of the negative prefix and suffix for STYLE, in
+   display columns (e.g. as returned by u8_strwidth()). */
 int
 fmt_neg_affix_width (const struct fmt_number_style *style)
 {
-  return ss_length (style->neg_prefix) + ss_length (style->neg_suffix);
+  return style->neg_prefix.width + style->neg_suffix.width;
 }
 
-
-/* Sets the number style for TYPE to have the given standard
-   PREFIX and SUFFIX, "-" as prefix suffix, an empty negative
-   suffix, DECIMAL as the decimal point character, and GROUPING
-   as the grouping character. */
-static void
-set_style (struct fmt_number_style *styles, enum fmt_type type,
-           const char *prefix, const char *suffix,
-           char decimal, char grouping)
-{
-  struct fmt_number_style *style;
-
-  assert (is_fmt_type (type));
-
-  style = &styles[type] ;
-
-  fmt_number_style_destroy (style);
-
-  ss_alloc_substring (&style->neg_prefix, ss_cstr ("-"));
-  ss_alloc_substring (&style->prefix, ss_cstr (prefix));
-  ss_alloc_substring (&style->suffix, ss_cstr (suffix));
-  style->decimal = decimal;
-  style->grouping = grouping;
-}
-
-/* Sets the decimal point character to DECIMAL. */
-void
-fmt_set_decimal (struct fmt_number_style *styles, char decimal)
-{
-  int grouping = decimal == '.' ? ',' : '.';
-  assert (decimal == '.' || decimal == ',');
-
-  set_style (styles, FMT_F, "", "", decimal, 0);
-  set_style (styles, FMT_E, "", "", decimal, 0);
-  set_style (styles, FMT_COMMA, "", "", decimal, grouping);
-  set_style (styles, FMT_DOT, "", "", grouping, decimal);
-  set_style (styles, FMT_DOLLAR, "$", "", decimal, grouping);
-  set_style (styles, FMT_PCT, "", "%", decimal, 0);
-}
-
 /* Returns the struct fmt_desc for the given format TYPE. */
 static const struct fmt_desc *
 get_fmt_desc (enum fmt_type type)
@@ -978,3 +1025,5 @@ get_fmt_desc (enum fmt_type type)
   assert (is_fmt_type (type));
   return &formats[type];
 }
+
+const struct fmt_spec F_8_0 = {FMT_F, 8, 0};
