@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-9, 2000, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000, 2006, 2007, 2009, 2010, 2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -15,24 +15,27 @@
    along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <config.h>
-#include "settings.h"
+
+#include "data/settings.h"
+
 #include <assert.h>
 #include <stdlib.h>
 #include <time.h>
-#include "format.h"
-#include "value.h"
-#include "xalloc.h"
-#include <libpspp/i18n.h>
-#include <libpspp/integer-format.h>
-#include <libpspp/message.h>
 
-#include "error.h"
+#include "data/case.h"
+#include "data/format.h"
+#include "data/value.h"
+#include "libpspp/i18n.h"
+#include "libpspp/integer-format.h"
+#include "libpspp/message.h"
+
+#include "gl/error.h"
+#include "gl/minmax.h"
+#include "gl/xalloc.h"
 
 #include "gettext.h"
 #define _(msgid) gettext (msgid)
 
-static int global_algorithm = ENHANCED;
-
 struct settings
 {
   /* Integer format used for IB and PIB input. */
@@ -47,115 +50,126 @@ struct settings
   /* Format of reals in output (SET WRB). */
   enum float_format output_float_format;
 
-  int *viewlength;
-  int *viewwidth;
+  int viewlength;
+  int viewwidth;
   bool safer_mode;
-  bool do_echo;
   bool include;
   int epoch;
-  bool errorbreak;
   bool route_errors_to_terminal;
   bool route_errors_to_listing;
   bool scompress;
   bool undefined;
   double blanks;
-  int mxwarns;
-  int mxerrs;
+  int max_messages[MSG_N_SEVERITIES];
   bool printback;
   bool mprint;
   int mxloops;
-  bool nulline;
-  char endcmd;
   size_t workspace;
   struct fmt_spec default_format;
   bool testing_mode;
 
   int cmd_algorithm;
-  int *algorithm;
+  int global_algorithm;
   int syntax;
 
-  struct fmt_number_style *styles;
+  struct fmt_settings *styles;
+
+  enum settings_output_devices output_routing[SETTINGS_N_OUTPUT_TYPES];
 };
 
 static struct settings the_settings = {
-    /* input_integer_format */
-  INTEGER_NATIVE,
-    /* input_float_format */
-  FLOAT_NATIVE_DOUBLE,
-    /* output_integer_format */
-  INTEGER_NATIVE,
-    /* output_float_format */
-  FLOAT_NATIVE_DOUBLE,
-    /* viewlength */
-  NULL,
-    /* viewwidth */
-  NULL,
-    /* safer_mode */
-  false,
-    /* do_echo */
-  false,
-    /* include */
-  true,
-    /* epoch */
-  -1,
-    /* errorbreak */
-  false,
-    /* route_errors_to_terminal */
-  true,
-    /* route_errors_to_listing */
-  true,
-    /* scompress */
-  true,
-    /* undefined */
-  true,
-    /* blanks */
-  SYSMIS,
-    /* mxwarns */
-  100,
-    /* mxerrs */
-  100,
-    /* printback */
-  true,
-    /* mprint */
-  true,
-    /* mxloops */
-  1,
-    /* nulline */
-  true,
-    /* endcmd */
-  '.',
-    /* workspace */
-  64L * 1024 * 1024,
-    /* default_format */
-  {FMT_F, 8, 2},
-    /* testing_mode */
-  false,
-    /* cmd_algorithm */
-  ENHANCED,
-    /* algorithm */
-  &global_algorithm,
-    /* syntax */
-  ENHANCED,
-    /* styles */
-  NULL
+  INTEGER_NATIVE,               /* input_integer_format */
+  FLOAT_NATIVE_DOUBLE,          /* input_float_format */
+  INTEGER_NATIVE,               /* output_integer_format */
+  FLOAT_NATIVE_DOUBLE,          /* output_float_format */
+  24,                           /* viewlength */
+  79,                           /* viewwidth */
+  false,                        /* safer_mode */
+  true,                         /* include */
+  -1,                           /* epoch */
+  true,                         /* route_errors_to_terminal */
+  true,                         /* route_errors_to_listing */
+  true,                         /* scompress */
+  true,                         /* undefined */
+  SYSMIS,                       /* blanks */
+
+  /* max_messages */
+  {
+    100,                        /* MSG_S_ERROR */
+    100,                        /* MSG_S_WARNING */
+    100                         /* MSG_S_NOTE */
+  },
+
+  true,                         /* printback */
+  true,                         /* mprint */
+  1,                            /* mxloops */
+  64L * 1024 * 1024,            /* workspace */
+  {FMT_F, 8, 2},                /* default_format */
+  false,                        /* testing_mode */
+  ENHANCED,                     /* cmd_algorithm */
+  ENHANCED,                     /* global_algorithm */
+  ENHANCED,                     /* syntax */
+  NULL,                         /* styles */
+
+  /* output_routing */
+  {SETTINGS_DEVICE_LISTING | SETTINGS_DEVICE_TERMINAL,
+   SETTINGS_DEVICE_LISTING | SETTINGS_DEVICE_TERMINAL,
+   0,
+   SETTINGS_DEVICE_LISTING | SETTINGS_DEVICE_TERMINAL}
 };
 
-static void init_viewport ( int *, int *);
-
+/* Initializes the settings module. */
 void
-settings_init (int *width, int *length)
+settings_init (void)
 {
-  init_viewport (width, length);
   settings_set_epoch (-1);
-  i18n_init ();
-  the_settings.styles = fmt_create ();
+  the_settings.styles = fmt_settings_create ();
+
+  settings_set_decimal_char (get_system_decimal ());
 }
 
+/* Cleans up the settings module. */
 void
 settings_done (void)
 {
-  fmt_done (the_settings.styles);
-  i18n_done ();
+  settings_destroy (&the_settings);
+}
+
+static void
+settings_copy (struct settings *dst, const struct settings *src)
+{
+  *dst = *src;
+  dst->styles = fmt_settings_clone (src->styles);
+}
+
+/* Returns a copy of the current settings. */
+struct settings *
+settings_get (void)
+{
+  struct settings *s = xmalloc (sizeof *s);
+  settings_copy (s, &the_settings);
+  return s;
+}
+
+/* Replaces the current settings by those in S.  The caller retains ownership
+   of S. */
+void
+settings_set (const struct settings *s)
+{
+  settings_destroy (&the_settings);
+  settings_copy (&the_settings, s);
+}
+
+/* Destroys S. */
+void
+settings_destroy (struct settings *s)
+{
+  if (s != NULL)
+    {
+      fmt_settings_destroy (s->styles);
+      if (s != &the_settings)
+        free (s);
+    }
 }
 
 /* Returns the floating-point format used for RB and RBHEX
@@ -222,35 +236,28 @@ settings_set_output_float_format ( enum float_format float_format)
 int
 settings_get_viewlength (void)
 {
-  return *the_settings.viewlength;
+  return the_settings.viewlength;
 }
 
 /* Sets the view length. */
 void
 settings_set_viewlength ( int viewlength_)
 {
-  *the_settings.viewlength = viewlength_;
+  the_settings.viewlength = viewlength_;
 }
 
 /* Screen width. */
 int
 settings_get_viewwidth(void)
 {
-  return *the_settings.viewwidth;
+  return the_settings.viewwidth;
 }
 
 /* Sets the screen width. */
 void
 settings_set_viewwidth ( int viewwidth_)
 {
-  *the_settings.viewwidth = viewwidth_;
-}
-
-static void
-init_viewport ( int  *width, int *length)
-{
-  the_settings.viewwidth = width;
-  the_settings.viewlength = length;
+  the_settings.viewwidth = viewwidth_;
 }
 
 /* Whether PSPP can erase and overwrite files. */
@@ -267,20 +274,6 @@ settings_set_safer_mode (void)
   the_settings.safer_mode = true;
 }
 
-/* Echo commands to the listing file/printer? */
-bool
-settings_get_echo (void)
-{
-  return the_settings.do_echo;
-}
-
-/* Set echo. */
-void
-settings_set_echo ( bool echo)
-{
-  the_settings.do_echo = echo;
-}
-
 /* If echo is on, whether commands from include files are echoed. */
 bool
 settings_get_include (void)
@@ -319,50 +312,6 @@ settings_set_epoch ( int epoch)
   assert (the_settings.epoch >= 0);
 }
 
-/* Does an error stop execution? */
-bool
-settings_get_errorbreak (void)
-{
-  return the_settings.errorbreak;
-}
-
-/* Sets whether an error stops execution. */
-void
-settings_set_errorbreak ( bool errorbreak)
-{
-  the_settings.errorbreak = errorbreak;
-}
-
-/* Route error messages to terminal? */
-bool
-settings_get_error_routing_to_terminal (void)
-{
-  return the_settings.route_errors_to_terminal;
-}
-
-/* Sets whether error messages should be routed to the
-   terminal. */
-void
-settings_set_error_routing_to_terminal ( bool route_to_terminal)
-{
-  the_settings.route_errors_to_terminal = route_to_terminal;
-}
-
-/* Route error messages to listing file? */
-bool
-settings_get_error_routing_to_listing (void)
-{
-  return the_settings.route_errors_to_listing;
-}
-
-/* Sets whether error messages should be routed to the
-   listing file. */
-void
-settings_set_error_routing_to_listing ( bool route_to_listing)
-{
-  the_settings.route_errors_to_listing = route_to_listing;
-}
-
 /* Compress system files by default? */
 bool
 settings_get_scompression (void)
@@ -406,46 +355,44 @@ settings_set_blanks ( double blanks)
   the_settings.blanks = blanks;
 }
 
-/* Maximum number of warnings + errors. */
+/* Returns the maximum number of messages to show of the given SEVERITY before
+   aborting.  (The value for MSG_S_WARNING is interpreted as maximum number of
+   warnings and errors combined.) */
 int
-settings_get_mxwarns (void)
+settings_get_max_messages (enum msg_severity severity)
 {
-  return the_settings.mxwarns;
+  assert (severity < MSG_N_SEVERITIES);
+  return the_settings.max_messages[severity];
 }
 
-/* Sets maximum number of warnings + errors. */
+/* Sets the maximum number of messages to show of the given SEVERITY before
+   aborting to MAX.  (The value for MSG_S_WARNING is interpreted as maximum
+   number of warnings and errors combined.)  In addition, in the case of 
+   warnings the special value of zero indicates that no warnings are to be
+   issued. 
+*/
 void
-settings_set_mxwarns ( int mxwarns)
+settings_set_max_messages (enum msg_severity severity, int max)
 {
-  the_settings.mxwarns = mxwarns;
-}
+  assert (severity < MSG_N_SEVERITIES);
 
-/* Maximum number of errors. */
-int
-settings_get_mxerrs (void)
-{
-  return the_settings.mxerrs;
-}
+  if (severity == MSG_S_WARNING)
+    {
+      if ( max == 0)
+	{
+	  msg (MW,
+	       _("MXWARNS set to zero.  No further warnings will be given even when potentially problematic situations are encountered."));
+	  msg_ui_disable_warnings (true);
+	}
+      else if ( the_settings.max_messages [MSG_S_WARNING] == 0)
+	{
+	  msg_ui_disable_warnings (false);
+	  the_settings.max_messages[MSG_S_WARNING] = max;
+	  msg (MW, _("Warnings re-enabled. %d warnings will be issued before aborting syntax processing."), max);
+	}
+    }
 
-/* Sets maximum number of errors. */
-void
-settings_set_mxerrs ( int mxerrs)
-{
-  the_settings.mxerrs = mxerrs;
-}
-
-/* Whether commands are written to the display. */
-bool
-settings_get_printback (void)
-{
-  return the_settings.printback;
-}
-
-/* Sets whether commands are written to the display. */
-void
-settings_set_printback ( bool printback)
-{
-  the_settings.printback = printback;
+  the_settings.max_messages[severity] = max;
 }
 
 /* Independent of get_printback, controls whether the commands
@@ -478,34 +425,6 @@ settings_set_mxloops ( int mxloops)
   the_settings.mxloops = mxloops;
 }
 
-/* Whether a blank line is a command terminator. */
-bool
-settings_get_nulline (void)
-{
-  return the_settings.nulline;
-}
-
-/* Set whether a blank line is a command terminator. */
-void
-settings_set_nulline ( bool nulline)
-{
-  the_settings.nulline = nulline;
-}
-
-/* The character used to terminate commands. */
-char
-settings_get_endcmd (void)
-{
-  return the_settings.endcmd;
-}
-
-/* Set the character used to terminate commands. */
-void
-settings_set_endcmd ( char endcmd)
-{
-  the_settings.endcmd = endcmd;
-}
-
 /* Approximate maximum amount of memory to use for cases, in
    bytes. */
 size_t
@@ -515,13 +434,12 @@ settings_get_workspace (void)
 }
 
 /* Approximate maximum number of cases to allocate in-core, given
-   that each case contains VALUE_CNT values. */
+   that each case has the format given in PROTO. */
 size_t
-settings_get_workspace_cases (size_t value_cnt)
+settings_get_workspace_cases (const struct caseproto *proto)
 {
-  size_t case_size = sizeof (union value) * value_cnt + 4 * sizeof (void *);
-  size_t case_cnt = MAX (settings_get_workspace () / case_size, 4);
-  return case_cnt;
+  size_t n_cases = settings_get_workspace () / case_get_cost (proto);
+  return MAX (n_cases, 4);
 }
 
 /* Set approximate maximum amount of memory to use for cases, in
@@ -568,14 +486,14 @@ settings_set_testing_mode ( bool testing_mode)
 enum behavior_mode
 settings_get_algorithm (void)
 {
-  return *the_settings.algorithm;
+  return the_settings.cmd_algorithm;
 }
 
 /* Set the algorithm option globally. */
 void
 settings_set_algorithm (enum behavior_mode mode)
 {
-  global_algorithm = mode;
+  the_settings.global_algorithm = the_settings.cmd_algorithm = mode;
 }
 
 /* Set the algorithm option for this command only */
@@ -583,14 +501,13 @@ void
 settings_set_cmd_algorithm ( enum behavior_mode mode)
 {
   the_settings.cmd_algorithm = mode;
-  the_settings.algorithm = &the_settings.cmd_algorithm;
 }
 
 /* Unset the algorithm option for this command */
 void
 unset_cmd_algorithm (void)
 {
-  the_settings.algorithm = &global_algorithm;
+  the_settings.cmd_algorithm = the_settings.global_algorithm;
 }
 
 /* Get the current syntax setting */
@@ -609,11 +526,10 @@ settings_set_syntax ( enum behavior_mode mode)
 
 
 
-/* Find the grouping characters in CC_STRING and set CC's
-   grouping and decimal members appropriately.  Returns true if
-   successful, false otherwise. */
+/* Find the grouping characters in CC_STRING and sets *GROUPING and *DECIMAL
+   appropriately.  Returns true if successful, false otherwise. */
 static bool
-find_cc_separators (const char *cc_string, struct fmt_number_style *cc)
+find_cc_separators (const char *cc_string, char *decimal, char *grouping)
 {
   const char *sp;
   int comma_cnt, dot_cnt;
@@ -635,52 +551,50 @@ find_cc_separators (const char *cc_string, struct fmt_number_style *cc)
 
   if (comma_cnt == 3)
     {
-      cc->decimal = '.';
-      cc->grouping = ',';
+      *decimal = '.';
+      *grouping = ',';
     }
   else
     {
-      cc->decimal = ',';
-      cc->grouping = '.';
+      *decimal = ',';
+      *grouping = '.';
     }
   return true;
 }
 
-/* Extracts a token from IN into a newly allocated AFFIX.  Tokens
-   are delimited by GROUPING.  The token is truncated to at most
-   FMT_STYLE_AFFIX_MAX characters.  Returns the first character
-   following the token. */
+/* Extracts a token from IN into a newly allocated string AFFIXP.  Tokens are
+   delimited by GROUPING.  Returns the first character following the token. */
 static const char *
-extract_cc_token (const char *in, int grouping, struct substring *affix)
+extract_cc_token (const char *in, int grouping, char **affixp)
 {
-  size_t ofs = 0;
-  ss_alloc_uninit (affix, FMT_STYLE_AFFIX_MAX);
+  char *out;
+
+  out = *affixp = xmalloc (strlen (in) + 1);
   for (; *in != '\0' && *in != grouping; in++)
     {
       if (*in == '\'' && in[1] == grouping)
         in++;
-      if (ofs < FMT_STYLE_AFFIX_MAX)
-        ss_data (*affix)[ofs++] = *in;
+      *out++ = *in;
     }
-  affix->length = ofs;
+  *out = '\0';
 
   if (*in == grouping)
     in++;
   return in;
 }
 
-
 /* Sets custom currency specifier CC having name CC_NAME ('A' through
    'E') to correspond to the settings in CC_STRING. */
 bool
 settings_set_cc (const char *cc_string, enum fmt_type type)
 {
-  struct fmt_number_style *cc = &the_settings.styles[type];
+  char *neg_prefix, *prefix, *suffix, *neg_suffix;
+  char decimal, grouping;
 
   assert (fmt_get_category (type) == FMT_CAT_CUSTOM);
 
   /* Determine separators. */
-  if (!find_cc_separators (cc_string, cc))
+  if (!find_cc_separators (cc_string, &decimal, &grouping))
     {
       msg (SE, _("%s: Custom currency string `%s' does not contain "
                  "exactly three periods or commas (or it contains both)."),
@@ -688,12 +602,18 @@ settings_set_cc (const char *cc_string, enum fmt_type type)
       return false;
     }
 
-  cc_string = extract_cc_token (cc_string, cc->grouping, &cc->neg_prefix);
-  cc_string = extract_cc_token (cc_string, cc->grouping, &cc->prefix);
-  cc_string = extract_cc_token (cc_string, cc->grouping, &cc->suffix);
-  cc_string = extract_cc_token (cc_string, cc->grouping, &cc->neg_suffix);
+  cc_string = extract_cc_token (cc_string, grouping, &neg_prefix);
+  cc_string = extract_cc_token (cc_string, grouping, &prefix);
+  cc_string = extract_cc_token (cc_string, grouping, &suffix);
+  cc_string = extract_cc_token (cc_string, grouping, &neg_suffix);
 
-  fmt_check_style (cc);
+  fmt_settings_set_style (the_settings.styles, type, decimal, grouping,
+                          neg_prefix, prefix, suffix, neg_suffix);
+
+  free (neg_suffix);
+  free (suffix);
+  free (prefix);
+  free (neg_prefix);
 
   return true;
 }
@@ -702,57 +622,68 @@ settings_set_cc (const char *cc_string, enum fmt_type type)
 int
 settings_get_decimal_char (enum fmt_type type)
 {
-  return fmt_get_style (the_settings.styles, type)->decimal;
+  return fmt_settings_get_style (the_settings.styles, type)->decimal;
 }
 
 void
 settings_set_decimal_char (char decimal)
 {
-  fmt_set_decimal (the_settings.styles, decimal);
+  fmt_settings_set_decimal (the_settings.styles, decimal);
 }
 
-
-
 /* Returns the number formatting style associated with the given
    format TYPE. */
 const struct fmt_number_style *
 settings_get_style (enum fmt_type type)
 {
   assert (is_fmt_type (type));
-  return &the_settings.styles[type];
+  return fmt_settings_get_style (the_settings.styles, type);
 }
 
-
 /* Returns a string of the form "$#,###.##" according to FMT,
    which must be of type FMT_DOLLAR.  The caller must free the
    string. */
 char *
 settings_dollar_template (const struct fmt_spec *fmt)
 {
-  const struct fmt_number_style *styles = the_settings.styles;
   struct string str = DS_EMPTY_INITIALIZER;
   int c;
   const struct fmt_number_style *fns ;
 
   assert (fmt->type == FMT_DOLLAR);
 
-  fns = fmt_get_style (styles, fmt->type);
+  fns = fmt_settings_get_style (the_settings.styles, fmt->type);
 
-  ds_put_char (&str, '$');
+  ds_put_byte (&str, '$');
   for (c = MAX (fmt->w - fmt->d - 1, 0); c > 0; )
     {
-      ds_put_char (&str, '#');
+      ds_put_byte (&str, '#');
       if (--c % 4 == 0 && c > 0)
         {
-          ds_put_char (&str, fns->grouping);
+          ds_put_byte (&str, fns->grouping);
           --c;
         }
     }
   if (fmt->d > 0)
     {
-      ds_put_char (&str, fns->decimal);
-      ds_put_char_multiple (&str, '#', fmt->d);
+      ds_put_byte (&str, fns->decimal);
+      ds_put_byte_multiple (&str, '#', fmt->d);
     }
 
   return ds_cstr (&str);
 }
+
+void
+settings_set_output_routing (enum settings_output_type type,
+                             enum settings_output_devices devices)
+{
+  assert (type < SETTINGS_N_OUTPUT_TYPES);
+  the_settings.output_routing[type] = devices;
+}
+
+enum settings_output_devices
+settings_get_output_routing (enum settings_output_type type)
+{
+  assert (type < SETTINGS_N_OUTPUT_TYPES);
+  return the_settings.output_routing[type] | SETTINGS_DEVICE_UNFILTERED;
+}
