@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-9, 2000, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1997-9, 2000, 2009, 2010, 2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -19,9 +19,9 @@
 
 #include <stddef.h>
 #include <stdbool.h>
-#include <data/dict-class.h>
-#include <data/missing-values.h>
-#include <data/val-type.h>
+#include "data/dict-class.h"
+#include "data/missing-values.h"
+#include "data/val-type.h"
 
 union value;
 
@@ -32,16 +32,10 @@ union value;
 struct variable *var_create (const char *name, int width);
 struct variable *var_clone (const struct variable *);
 void var_destroy (struct variable *);
-struct variable *var_create_internal (int case_idx, int width);
-
 
 /* Variable names. */
-#define VAR_NAME_LEN 64 /* Maximum length of variable name, in bytes. */
-
 const char *var_get_name (const struct variable *);
 void var_set_name (struct variable *, const char *);
-bool var_is_valid_name (const char *, bool issue_error);
-bool var_is_plausible_name (const char *name, bool issue_error);
 enum dict_class var_get_dict_class (const struct variable *);
 
 int compare_vars_by_name (const void *, const void *, const void *);
@@ -104,7 +98,8 @@ struct fmt_spec var_default_formats (int width);
 /* Variable labels. */
 const char *var_to_string (const struct variable *);
 const char *var_get_label (const struct variable *);
-void var_set_label (struct variable *, const char *);
+bool var_set_label (struct variable *, const char *label,
+                    const char *dict_encoding, bool issue_warning);
 void var_clear_label (struct variable *);
 bool var_has_label (const struct variable *);
 
@@ -168,16 +163,14 @@ void var_clear_aux (struct variable *);
 void *var_detach_aux (struct variable *);
 void var_dtor_free (struct variable *);
 
-/* Observed categorical values. */
-struct cat_vals *var_get_obs_vals (const struct variable *);
-void var_set_obs_vals (const struct variable *, struct cat_vals *);
-bool var_has_obs_vals (const struct variable *);
-
 /* Custom attributes. */
 struct attrset *var_get_attributes (const struct variable *);
 void var_set_attributes (struct variable *, const struct attrset *);
 bool var_has_attributes (const struct variable *);
 
+/* Encoding. */
+const char *var_get_encoding (const struct variable *);
+
 /* Function types. */
 typedef bool var_predicate_func (const struct variable *);
 
