@@ -1,5 +1,5 @@
 /* PSPP - a program for statistical analysis.
-   Copyright (C) 1997-2004, 2006 Free Software Foundation, Inc.
+   Copyright (C) 1997-2004, 2006, 2010, 2011 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 
 #include <config.h>
 
-#include <language/data-io/data-writer.h>
+#include "language/data-io/data-writer.h"
 
 #include <assert.h>
 #include <errno.h>
@@ -24,16 +24,16 @@
 #include <stdlib.h>
 #include <sys/stat.h>
 
-#include <data/file-name.h>
-#include <data/make-file.h>
-#include <language/data-io/file-handle.h>
-#include <libpspp/assertion.h>
-#include <libpspp/integer-format.h>
-#include <libpspp/message.h>
-#include <libpspp/str.h>
+#include "data/file-name.h"
+#include "data/make-file.h"
+#include "language/data-io/file-handle.h"
+#include "libpspp/assertion.h"
+#include "libpspp/integer-format.h"
+#include "libpspp/message.h"
+#include "libpspp/str.h"
 
-#include "minmax.h"
-#include "xalloc.h"
+#include "gl/minmax.h"
+#include "gl/xalloc.h"
 
 #include "gettext.h"
 #define _(msgid) gettext (msgid)
@@ -66,12 +66,11 @@ dfm_open_writer (struct file_handle *fh)
   w = xmalloc (sizeof *w);
   w->fh = fh_ref (fh);
   w->lock = lock;
-  w->rf = replace_file_start (fh_get_file_name (w->fh), "wb",
-                              (S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP
-                               | S_IROTH | S_IWOTH), &w->file, NULL);
+  w->rf = replace_file_start (fh_get_file_name (w->fh), "wb", 0666,
+                              &w->file, NULL);
   if (w->rf == NULL)
     {
-      msg (ME, _("An error occurred while opening \"%s\" for writing "
+      msg (ME, _("An error occurred while opening `%s' for writing "
                  "as a data file: %s."),
            fh_get_file_name (w->fh), strerror (errno));
       dfm_close_writer (w);
@@ -188,7 +187,7 @@ dfm_close_writer (struct dfm_writer *w)
       ok = !dfm_write_error (w) && !fn_close (file_name, w->file);
 
       if (!ok)
-        msg (ME, _("I/O error occurred writing data file \"%s\"."), file_name);
+        msg (ME, _("I/O error occurred writing data file `%s'."), file_name);
 
       if (ok ? !replace_file_commit (w->rf) : !replace_file_abort (w->rf))
         ok = false;
