@@ -19,9 +19,9 @@
 
 
 #include <gtk/gtk.h>
-#include <glade/glade.h>
+#include "psppire-data-window.h"
 
 
-void descriptives_dialog (GObject *o, gpointer data);
+void descriptives_dialog (PsppireDataWindow * data);
 
 #endif
