@@ -19,21 +19,12 @@
    dictionary in  GTK widgets.
 */
 
+#ifndef DICT_DISPLAY 
+#define DICT_DISPLAY 1
+
 #include <gtk/gtk.h>
 
 #include "psppire-selector.h"
-#include "psppire-dict.h"
-#include <data/variable.h>
-
-/* Sets up TREEVIEW to display the variables of DICT.
-   MODE is the selection mode for TREEVIEW.
-   PREDICATE determines which variables should be visible, or NULL if
-   all are to be visible.
- */
-void attach_dictionary_to_treeview (GtkTreeView *treeview, PsppireDict *dict,
-				    GtkSelectionMode mode,
-				    var_predicate_func *predicate
-				    );
 
 
 /* A SelectItemsFunc function for GtkTreeView widgets */
@@ -43,7 +34,6 @@ void insert_source_row_into_tree_view (GtkTreeIter source_iter,
 				       gpointer data
 				       );
 
-
 /* A SelectItemsFunc function for GtkEntry widgets */
 void insert_source_row_into_entry (GtkTreeIter source_iter,
 				   GtkWidget *dest,
@@ -57,7 +47,4 @@ void insert_source_row_into_entry (GtkTreeIter source_iter,
 gboolean is_currently_in_entry (GtkTreeModel *model, GtkTreeIter *iter,
 				PsppireSelector *selector);
 
-
-void get_base_model (GtkTreeModel *top_model, GtkTreeIter *top_iter,
-		     GtkTreeModel **model, GtkTreeIter *iter
-		     );
+#endif
