@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2007  Free Software Foundation
+   Copyright (C) 2007, 2010  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -305,7 +305,7 @@ configure_event_callback (GtkDialog *dialog,
 
   g_object_get (dialog, "name", &base, NULL);
 
-  psppire_conf_save_window_geometry (conf, base, event);
+  psppire_conf_save_window_geometry (conf, base, GTK_WINDOW (dialog));
 
   return FALSE;
 }
@@ -481,7 +481,7 @@ connect_notify_signal (GtkWidget *w, gpointer data)
 
       while ((col = gtk_tree_view_get_column (tv, i++)))
 	{
-	  GList *renderers = gtk_tree_view_column_get_cell_renderers (col);
+	  GList *renderers = gtk_cell_layout_get_cells (GTK_CELL_LAYOUT (col));
 	  GList *start = renderers;
 	  while (renderers)
 	    {
