@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2009  Free Software Foundation
+   Copyright (C) 2009, 2010  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -203,19 +203,6 @@ on_realize (GtkWindow *window, gpointer data)
 }
 
 
-static gboolean
-save_geometry (GtkWidget *window, GdkEvent *event, gpointer data)
-{
-  const gchar *base = G_OBJECT_TYPE_NAME (window);
-
-  PsppireConf *conf = psppire_conf_new ();
-
-  psppire_conf_save_window_geometry (conf, base, event);
-
-  return FALSE;
-}
-
-
 static void
 psppire_window_finalize (GObject *object)
 {
@@ -256,7 +243,8 @@ psppire_window_class_init (PsppireWindowClass *class)
     g_param_spec_string ("filename",
 		       "File name",
 		       "The name of the file associated with this window, if any",
-			 "Untitled",
+			 /* TRANSLATORS: This will form a filename.  Please avoid whitespace. */
+			 _("Untitled"),
 			 G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
 
   g_unichar_to_utf8 (0x2014, mdash);
@@ -382,6 +370,13 @@ on_delete (PsppireWindow *w, GdkEvent *event, gpointer user_data)
 {
   PsppireWindowRegister *reg = psppire_window_register_new ();
 
+  const gchar *base = G_OBJECT_TYPE_NAME (w);
+
+  PsppireConf *conf = psppire_conf_new ();
+
+  psppire_conf_save_window_geometry (conf, base, GTK_WINDOW (w));
+
+
   if ( w->dirty )
     {
       gint response = psppire_window_query_save (w);
@@ -435,12 +430,6 @@ psppire_window_init (PsppireWindow *window)
 
   g_object_set (window, "icon-name", "psppicon", NULL);
 
-  g_signal_connect (window, "configure-event",
-		    G_CALLBACK (save_geometry), window);
-
-  g_signal_connect (window, "window-state-event",
-		    G_CALLBACK (save_geometry), window);
-
   g_signal_connect (window, "realize",
 		    G_CALLBACK (on_realize), window);
 
@@ -477,7 +466,7 @@ psppire_window_query_save (PsppireWindow *se)
 			    GTK_DIALOG_MODAL,
 			    GTK_MESSAGE_WARNING,
 			    GTK_BUTTONS_NONE,
-			    _("Save the changes to \"%s\" before closing?"),
+			    _("Save the changes to `%s' before closing?"),
 			    fn);
   g_free (fn);
 
