@@ -1,5 +1,5 @@
 /* PSPPIRE - a graphical user interface for PSPP.
-   Copyright (C) 2008  Free Software Foundation
+   Copyright (C) 2008, 2010  Free Software Foundation
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
 #ifndef TEXT_DATA_IMPORT_DIALOG_H
 #define TEXT_DATA_IMPORT_DIALOG_H
 
-#include <glib-object.h>
+#include <gtk/gtk.h>
 
-void text_data_import_assistant (GObject *o, gpointer data);
+void text_data_import_assistant (GtkWindow *parent_window);
 
 #endif
